"""Models for the flipbook app."""
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from filer.fields.file import FilerFileField
from filer.fields.folder import FilerFolderField
from filer.models.imagemodels import Image


class Flipbook(models.Model):
    """
    Model, which holds information about a flipbook and it's contents.

    :user: User, who owns the flipbook.
    :title: Flipbook title.
    :slug: Title in slug format.
    :folder: Linked folder of the filer app.
    :is_published: True if the book is published or not.
    :download: Flipbook file to downlod.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='flipbooks',
        blank=True, null=True,
    )

    title = models.CharField(
        max_length=100,
        verbose_name=_('Title'),
    )

    slug = models.SlugField(
        max_length=100,
        verbose_name=_('Slug'),
    )

    folder = FilerFolderField(
        verbose_name=_('Folder'),
    )

    is_published = models.BooleanField(
        verbose_name=_('Is published'),
        default=False,
    )

    download = FilerFileField(
        verbose_name=_('Download'),
        blank=True, null=True,
    )

    def __unicode__(self):
        return u'{0}'.format(self.title)

    def get_absolute_url(self):
        return reverse('flipbook_detail', kwargs={'slug': self.slug})

    def get_folder_images(self):
        """
        Returns a set of images, which have been placed in this folder.

        """
        qs_files = self.folder.files.instance_of(Image)
        return qs_files.filter(is_public=True)



"""Admin registration for the ``flipbook`` app."""
from django.contrib import admin

import models


class FlipbookAdmin(admin.ModelAdmin):
    list_display = ('user', 'title', 'slug', 'is_published')
    prepopulated_fields = {'slug': ('title', )}


admin.site.register(models.Flipbook, FlipbookAdmin)



"""URLs for the ``flipbook`` app."""
from django.conf.urls import patterns, url

from . import views


urlpatterns = patterns(
    '',
    url(r'^(?P<slug>[\w-]+)/$',
        views.FlipbookDetailView.as_view(),
        name='flipbook_detail'),
    url(r'^$',
        views.FlipbookListView.as_view(),
        name='flipbook_list'),
)



"""Factories for the ``flipbook`` app."""
import factory

from filer.models import Folder

from ..models import Flipbook


class FolderFactory(factory.DjangoModelFactory):
    FACTORY_FOR = Folder

    name = 'Test Folder'


class FlipbookFactory(factory.DjangoModelFactory):
    FACTORY_FOR = Flipbook

    title = factory.Sequence(lambda i: 'name {}'.format(i))
    slug = factory.LazyAttribute(lambda a: a.title.replace(' ', '-'))
    folder = factory.SubFactory(FolderFactory)
    is_published = True



"""Settings that need to be set in order to run the tests."""
import os

DEBUG = True

SITE_ID = 1

DJANGO_PROJECT_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..'))


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'flipbook.tests.urls'

MEDIA_URL = '/media/'
STATIC_URL = '/static/'
MEDIA_ROOT = os.path.join(DJANGO_PROJECT_ROOT, '../media')
STATIC_ROOT = os.path.join(DJANGO_PROJECT_ROOT, '../static')
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(DJANGO_PROJECT_ROOT, 'static'),
)

TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), '../templates'),
)

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.dirname(__file__), 'coverage')
COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'django_nose',
    'filer',
    'mptt',
    'easy_thumbnails',
]

INTERNAL_APPS = [
    'flipbook',
    'flipbook.tests.test_app',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS
COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS

SECRET_KEY = 'foobar'

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)



"""Tests for the models of the ``flipbook`` app."""
from django.test import TestCase

from .factories import FlipbookFactory


class FlipbookTestCase(TestCase):
    """Tests for the ``Flipbook`` model."""
    longMessage = True

    def setUp(self):
        self.flipbook = FlipbookFactory()

    def test_model(self):
        self.assertTrue(self.flipbook.pk, msg=(
            'Should be able to instantiate and save the object.'))

    def test_get_folder_images(self):
        self.assertEqual(self.flipbook.get_folder_images().count(), 0, msg=(
            'Should return an empty image list.'))



"""Views for the ``flipbook`` app."""
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.views.generic import DetailView, ListView

from .models import Flipbook


class FlipbookDetailView(DetailView):
    """View to display a flipbook."""
    model = Flipbook

    def dispatch(self, request, *args, **kwargs):
        self.kwargs = kwargs
        self.object = self.get_object()
        if not self.object.is_published:
            return HttpResponseRedirect(reverse('flipbook_list'))
        return super(FlipbookDetailView, self).dispatch(
            request, *args, **kwargs)


class FlipbookListView(ListView):
    """View to display a list of published flipbooks."""
    def get_queryset(self):
        return Flipbook.objects.filter(is_published=True)



"""Tests for the views of the ``flipbook`` app."""
from django.core.urlresolvers import reverse
from django.test import TestCase

from django_libs.tests.mixins import ViewRequestFactoryTestMixin

from .factories import FlipbookFactory
from .. import views


class FlipbookDetailViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Test case for the FlipbookDetailView view."""
    view_class = views.FlipbookDetailView

    def setUp(self):
        self.flipbook = FlipbookFactory()

    def get_view_name(self):
        return 'flipbook_detail'

    def get_view_kwargs(self):
        return {'slug': self.flipbook.slug}

    def test_view(self):
        self.is_callable()
        self.flipbook.is_published = False
        self.flipbook.save()
        self.redirects(reverse('flipbook_list'))


class FlipbookListViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Test case for the FlipbookListView view."""
    view_class = views.FlipbookListView

    def setUp(self):
        self.flipbook = FlipbookFactory()

    def get_view_name(self):
        return 'flipbook_list'

    def test_view(self):
        self.is_callable()
